from abc import ABC, abstractmethod
from typing import Optional


class ModuleABC(ABC):

    @abstractmethod
    def __init__(self, priorities: dict[type, int], settings_types: list[Optional[type]]):
        self._priorities = priorities
        self._success = True
        self._settings_types = settings_types

    @property
    def success(self) -> bool:
        return self._success

    @property
    def settings_types(self) -> list[type]:
        return self._settings_types

    def get_priority(self, t: type) -> int:
        if t not in self._priorities:
            raise Exception(f'Priority for {t} not found!')
        return self._priorities[t]

    def stop_propagation(self):
        self._success = False
