from cpl_core.logging import LoggerABC

from gismo_data.abc.migration_abc import MigrationABC
from gismo_data.db_context import DBContext


class Migration_0_3(MigrationABC):

    def __init__(self, logger: LoggerABC, db: DBContext):
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, 'Running upgrade')

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Clients` (
                `ClientId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordClientId` BIGINT NOT NULL,
                `SentMessageCount` BIGINT NOT NULL DEFAULT 0,
                `ReceivedMessageCount` BIGINT NOT NULL DEFAULT 0,
                `DeletedMessageCount` BIGINT NOT NULL DEFAULT 0,
                `ReceivedCommandsCount` BIGINT NOT NULL DEFAULT 0,
                `MovedUsersCount` BIGINT NOT NULL DEFAULT 0,
                `ServerId` BIGINT,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`ServerId`) REFERENCES Servers(`ServerId`),
                PRIMARY KEY(`ClientId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `KnownUsers` (
                `KnownUserId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`KnownUserId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedServers` (
                `JoinId` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `JoinedOn` DATETIME(6) NOT NULL,
                `LeavedOn` DATETIME(6),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                PRIMARY KEY(`JoinId`)
            );
            """)
        )

    def downgrade(self):
        self._cursor.execute('DROP TABLE `Clients`;')
        self._cursor.execute('DROP TABLE `KnownUsers`;')
        self._cursor.execute('DROP TABLE `UserJoinedServers`;')
