from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContext
from cpl_core.logging import LoggerABC
from discord import VoiceChannel
from discord.ext import commands
from discord.ext.commands import Context

from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from gismo_core.abc.command_abc import CommandABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.configuration.server_settings import ServerSettings
from gismo_data.abc.client_repository_abc import ClientRepositoryABC
from modules.base.base_settings import BaseSettings


class AFKCommandService(CommandABC):

    def __init__(
            self,
            logger: LoggerABC,
            config: ConfigurationABC,
            message_service: MessageServiceABC,
            clients: ClientRepositoryABC,
            db: DatabaseContext,
            bot: BotServiceABC,
            client_utils: ClientUtilsServiceABC
    ):
        CommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._clients = clients
        self._db = db
        self._bot = bot
        self._client_utils = client_utils

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    async def afk(self, ctx: Context):
        self._logger.debug(__name__, f'Received command afk {ctx}')
        self._client_utils.received_command(ctx.guild.id)
        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        server_settings: ServerSettings = self._config.get_configuration(f'DSERVER_{ctx.guild.id}')

        if ctx.author.voice is None or ctx.author.voice.channel is None:
            await self._message_service.send_ctx_msg(ctx, settings.afk_command_channel_missing_message)
            self._logger.trace(__name__, f'Finished afk command')
            return

        self._bot.loop.create_task(self._message_service.send_ctx_msg(ctx, settings.afk_command_move_message))
        channel: VoiceChannel = ctx.guild.get_channel(settings.afk_command_channel_id)
        try:
            await ctx.author.move_to(channel)
            self._client_utils.moved_user(ctx.guild.id)
        except Exception as e:
            self._logger.error(__name__, f'Cannot move user {ctx.author.id} to channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, server_settings.bot_has_no_permission_message)

        self._logger.trace(__name__, f'Finished afk command')
