import asyncio

from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from discord.ext import commands
from discord.ext.commands import Context

from gismo_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from gismo_core.abc.command_abc import CommandABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.configuration.server_settings import ServerSettings
from modules.base.base_settings import BaseSettings
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PurgeCommandService(CommandABC):

    def __init__(
            self,
            logger: LoggerABC,
            config: ConfigurationABC,
            message_service: MessageServiceABC,
            permissions: PermissionServiceABC,
            client_utils: ClientUtilsServiceABC
    ):
        CommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._permissions = permissions
        self._client_utils = client_utils

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    async def purge(self, ctx: Context):
        self._logger.debug(__name__, f'Received command purge {ctx}')
        self._client_utils.received_command(ctx.guild.id)

        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        server_settings: ServerSettings = self._config.get_configuration(f'DSERVER_{ctx.guild.id}')

        if not self._permissions.is_member_moderator(ctx.author):
            await self._message_service.send_ctx_msg(ctx, settings.no_permissions_message)
            self._logger.trace(__name__, f'Finished purge command')
            return

        await self._message_service.send_ctx_msg(ctx, settings.purge_message)
        await asyncio.sleep(server_settings.message_delete_timer)
        try:
            await ctx.channel.purge()
        except Exception as e:
            self._logger.error(__name__, f'Cannot purge channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, server_settings.bot_has_no_permission_message)

        self._logger.trace(__name__, f'Finished purge command')
